import 'package:date_field/date_field.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:intl/intl.dart';
import 'package:syncfusion_flutter_datagrid/datagrid.dart';
import 'package:xcommonwidgets/xcommonwidgets.dart';
import 'package:xtoolspackage/xtoolspackage.dart';

import '../XFDataItem.dart';
import 'XF_View.dart';
import 'XFx.dart';

class XGridDataSource<T extends XFDataItem> extends DataGridSource {
  XGridDataSource(List<T> items, XView this.view) {
    list.addAll(items);
    dataGridRows = list.map<XDataGridRow>((e) => getDataGridRow(e)).toList();
  }
  late XView view;

  late List<T> list = [];
  dynamic cellInEdit_Value;
  List<XDataGridRow> dataGridRows = [];

  XDataGridRow getDataGridRow(XFDataItem item) {
    return XDataGridRow(item,
        cells: view.cols.map((e) {
          return e.getXDGCell(item.getProp(e.colKey));
        }).toList());
  }

  @override
  List<XDataGridRow> get rows => dataGridRows;
  @nonVirtual
  @override
  void onCellCancelEdit(DataGridRow dataGridRow, RowColumnIndex rowColumnIndex, GridColumn column) {
    xOnCellCancelEdit(dataGridRow, rowColumnIndex, column);
  }

  void xOnCellCancelEdit(DataGridRow dataGridRow, RowColumnIndex rowColumnIndex, GridColumn column) {
    // handle the cancel editing code here
    cellInEdit_Value = null;
  }

  @override
  bool onCellBeginEdit(DataGridRow dataGridRow, RowColumnIndex rowColumnIndex, GridColumn column) {
    // if (column.columnName == 'id') {
    //   // Return false, to restrict entering into the editing.
    //   return false;
    // } else {
    //   return true;
    // }

    //
    // questo serve a far si che la logica che decide il submit del valore della cella si scateni o no
    cellInEdit_Value = dataGridRow.getCells()[rowColumnIndex.columnIndex].value;
    return true;
  }

  @nonVirtual
  @override
  Future<bool> canSubmitCell(DataGridRow dataGridRow, RowColumnIndex rowColumnIndex, GridColumn column) {
    return xCanSubmitCell(dataGridRow, rowColumnIndex, column);
  }

  Future<bool> xCanSubmitCell(DataGridRow dataGridRow, RowColumnIndex rowColumnIndex, GridColumn column) {
    // if (column.columnName == 'id' && newCellValue == null) {
    //   // Return false, to retain in edit mode.
    //   // To avoid null value for cell
    //   return false;
    // } else {
    return Future.value(true);
    //}
  }

  @nonVirtual
  @override
  Future<void> onCellSubmit(DataGridRow dataGridRow, RowColumnIndex rowColumnIndex, GridColumn column) {
    return xOnCellSubmit(dataGridRow as XDataGridRow, rowColumnIndex, column);
  }

  Future<void> xOnCellSubmit(XDataGridRow dataGridRow, RowColumnIndex rowColumnIndex, GridColumn column) {
    set_ItemPropValue(rowColumnIndex, column, cellInEdit_Value);
    dataGridRow.getCells()[rowColumnIndex.columnIndex] = DataGridCell<dynamic>(columnName: column.columnName, value: cellInEdit_Value);
    return Future.value(null);
  }

  dynamic get_ItemPropValue(RowColumnIndex rowColumnIndex, GridColumn column) {
    return list[rowColumnIndex.rowIndex].getProp(column.columnName);
  }

  set_ItemPropValue(RowColumnIndex rowColumnIndex, GridColumn column, dynamic value) {
    // print("object" + list[rowColumnIndex.rowIndex][column.columnName]);
    // print("object value" + value);
    var xfdi = (list[rowColumnIndex.rowIndex]);
    // print("object xfdi " + xfdi.toString());

    xfdi[column.columnName] = value;
    // print("object xfdi va" + xfdi[column.columnName]);
  }

  @override
  DataGridRowAdapter? buildRow(DataGridRow row) {
    return xBuildRow(row as XDataGridRow);
  }

  DataGridRowAdapter? xBuildRow(XDataGridRow row) {
    return DataGridRowAdapter(
        cells: row.getCells().map<Widget>((dataGridCell) {
      return Container(
        color: view.cols.firstWhere((element) => element.colKey == dataGridCell.columnName).xmodelXprop.col_Color.withAlpha(60),
        alignment: Alignment.centerLeft,
        padding: EdgeInsets.symmetric(horizontal: 10.0),
        child: xBuilder_XCell(dataGridCell, row),
      );
    }).toList());
  }

  Widget? xBuilder_XCell(DataGridCell<dynamic> item, XDataGridRow xGridDataRow) {
    print("xBuilder_XCell");
    if (item.value == null) {
      return Text("", style: XStyles.xStyTextForSubLabel(Colors.white));
    } else {
      if (item.value!.runtimeType == String) {
        return Text(item.value, style: XStyles.xStyTextForSubLabel(Colors.white));
      } else if (item.value!.runtimeType == bool && item.value == true) {
        return Container(alignment: Alignment.center, child: Icon(Icons.medical_services_outlined));
      } else if (item.value!.runtimeType == DateTime) {
        return Text(XUtils.dateToString(item.value, format: "dd MMM yyyy ", noDateLabel: ""), style: XStyles.xStyTextForSubLabel(Colors.white));
      } else {
        return Text(item.value.toString(), style: XStyles.xStyTextForSubLabel(Colors.white));
      }
    }
  }

  // EDIT     // EDIT     // EDIT     // EDIT     // EDIT     // EDIT     // EDIT     // EDIT     // EDIT     // EDIT
  @nonVirtual
  @override
  Widget? buildEditWidget(DataGridRow dataGridRow, RowColumnIndex rowColumnIndex, GridColumn column, CellSubmit submitCell) {
    print("buildEditWidget");
    var gridCell = dataGridRow.getCells()[rowColumnIndex.columnIndex];

    return xBuilder_XCell_InEdit(gridCell, dataGridRow as XDataGridRow, rowColumnIndex, column, submitCell);
  }

  /// settare il value in cellInEdit_Value e chiamare  submitCell()
  Widget? xBuilder_XCell_InEdit(DataGridCell gridCell, XDataGridRow dataGridRow, RowColumnIndex rowColumnIndex, GridColumn column, CellSubmit submitCell) {
    if (gridCell.value!.runtimeType == String) {
      return XTextFormField(
          autoFocus: true,
          initialValue: gridCell.value,
          borderVisible: false,
          onChanged: (value) {
            cellInEdit_Value = value;
          },
          onFieldSubmitted: (value) {
            var oldVal = get_ItemPropValue(rowColumnIndex, column);
            print("value:" + value!);
            print("oldVal:" + oldVal);
            if (oldVal != cellInEdit_Value) {
              submitCell();
            }
          });
    } else if (gridCell.value!.runtimeType == DateTime) {
      return DateTimeFormField(
        decoration: InputDecoration(
          labelStyle: XStyles.xStyTextForSubLabel(Colors.white),
          hintStyle: XStyles.xStyTextForSubLabel(Colors.white),
          errorStyle: TextStyle(color: Colors.redAccent),
        ),
        onSaved: (date) {
          submitCell();
        },
        onDateSelected: (date) => cellInEdit_Value = date,
        initialValue: gridCell.value,
        dateFormat: DateFormat("dd MMMM yyyy"),
        dateTextStyle: XStyles.xStyTextForSubLabel(Colors.white),
        initialEntryMode: DatePickerEntryMode.input,
        initialDatePickerMode: DatePickerMode.year,
        mode: DateTimeFieldPickerMode.date,
        autovalidateMode: AutovalidateMode.onUserInteraction,
      );
    } else if (gridCell.value!.runtimeType == double) {
      return Text(gridCell.value.toString(), style: XStyles.xStyTextForSubLabel(Colors.white));
    } else {
      return Container();
    }
  }
}
